require "spec_helper"

RSpec.describe TextEditor::Command::CursorUp do
  include CommandHelper

  describe "#run" do
    context "with an empty buffer" do
      before do
        expect_buffer("")
        expect_cursor(0, 0)
      end

      it "does not move cursor" do
        subject.run
        expect_buffer("")
        expect_cursor(0, 0)
      end
    end

    context "with a one line buffer" do
      before { update_buffer("ab") }

      it "does not move cursor" do
        subject.run
        expect_buffer("ab")
        expect_cursor(0, 0)
      end
    end

    context "with a buffer containing multiple lines" do
      before do
        update_buffer("ab\ncd")
        update_cursor(1, 1)
      end

      it "moves up one line" do
        subject.run
        expect_buffer("ab\ncd")
        expect_cursor(0, 1)
      end
    end

    context "with a buffer containing different length lines" do
      before do
        update_buffer("a\nbcd")
        update_cursor(1, 2)
      end

      it "moves up one line and clamps to the end" do
        subject.run
        expect_buffer("a\nbcd")
        expect_cursor(0, 1)
      end
    end
  end
end
