require "spec_helper"

RSpec.describe TextEditor::Command::Resolver do
  subject { described_class.new(editor) }

  let(:editor) { double("editor") }
  let(:fake) { Class.new(namespace) }
  let(:namespace) { TextEditor::Command }

  before { namespace.const_set(:FakeCommand, fake) }
  after { namespace.send(:remove_const, :FakeCommand) }

  describe "#method_missing" do
    it "runs a command instance" do
      expect { subject.fake_command }.to raise_error(NotImplementedError)
    end

    it "returns self" do
      fake.send(:define_method, :run) { true }
      expect(subject.fake_command).to eq(subject)
    end

    it "raises NoMethodError for invalid commands" do
      expect { subject.invalid_command }.to raise_error(NoMethodError)
      expect { subject.invalid_command! }.to raise_error(NoMethodError)
      expect { subject.invalid_command? }.to raise_error(NoMethodError)
    end
  end

  describe "#resolve" do
    it "finds existing string commands" do
      actual = subject.resolve("fake_command")
      expect(actual).to eq(fake)
    end

    it "finds existing symbol commands" do
      actual = subject.resolve(:fake_command)
      expect(actual).to eq(fake)
    end

    it "returns nil for missing commands" do
      actual = subject.resolve(:invalid)
      expect(actual).to be_nil
    end
  end

  describe "#respond_to?" do
    it "only returns true for defined commands" do
      actual = subject.respond_to?(:fake_command)
      expect(actual).to eq(true)

      actual = subject.respond_to?(:invalid_command)
      expect(actual).to eq(false)
    end
  end
end
