require "spec_helper"

RSpec.describe TextEditor::Command do
  subject { described_class.new(editor) }

  let(:editor) { double("editor") }

  describe ".call" do
    it "delegates to new instance" do
      args = [1, 2, 3]
      block = proc { true }
      command = double("command")
      expect(command).to receive(:run).with(*args, &block)
      expect(described_class).to receive(:new).with(editor).and_return(command)
      described_class.call(editor, *args, &block)
    end
  end

  describe "#buffer" do
    it "delegates to editor.window" do
      expect(editor).to receive_message_chain(:window, :buffer)
      subject.buffer
    end
  end

  describe "#column" do
    it "delegates to editor.window.cursor" do
      expect(editor).to receive_message_chain(:window, :cursor, :column)
      subject.column
    end
  end

  describe "#content" do
    it "returns buffer content for the current line" do
      line = "test"
      buffer = double("buffer")
      expect(buffer).to receive(:line).with(line)
      expect(subject).to receive(:line).and_return(line)
      expect(subject).to receive(:buffer).and_return(buffer)
      subject.content
    end
  end

  describe "#cursor" do
    it "delegates to editor.window" do
      expect(editor).to receive_message_chain(:window, :cursor)
      subject.cursor
    end
  end

  describe "#line" do
    it "delegates to editor.window.line" do
      expect(editor).to receive_message_chain(:window, :cursor, :line)
      subject.line
    end
  end

  describe "#run" do
    it "must be implemented" do
      expect { subject.run }.to raise_error(NotImplementedError)
    end
  end

  describe "#scan" do
    it "returns a string scanner for the content of the current line" do
      expect(subject).to receive(:content).and_return("test")
      expect(StringScanner).to receive(:new).with("test").and_return(:example)
      expect(subject.scan).to eq(:example)
    end
  end

  describe "#state" do
    it "delegates to editor.window" do
      expect(editor).to receive_message_chain(:window, :state)
      subject.state
    end
  end

  describe "#window" do
    it "delegates to editor" do
      expect(editor).to receive(:window)
      subject.window
    end
  end
end
