require "spec_helper"

RSpec.describe TextEditor::Cursor do
  subject { described_class.new(line, column) }

  let(:line) { 1 }
  let(:column) { 2 }

  describe "#column" do
    it "returns current column" do
      expect(subject.column).to eq(2)
    end
  end

  describe "#down" do
    it "moves cursor down" do
      actual = subject.down
      expect(actual.position).to eq([2, 2])

      actual = subject.down(2)
      expect(actual.position).to eq([3, 2])
    end
  end

  describe "#initialize" do
    it "sets line and column to 0 by default" do
      expect(described_class.new.position).to eq([0, 0])
    end
  end

  describe "#left" do
    it "moves cursor left" do
      actual = subject.left
      expect(actual.position).to eq([1, 1])

      actual = subject.left(2)
      expect(actual.position).to eq([1, 0])

      actual = subject.left(3)
      expect(actual.position).to eq([1, 0])
    end
  end

  describe "#line" do
    it "returns current line" do
      expect(subject.line).to eq(1)
    end
  end

  describe "#move" do
    it "moves cursor" do
      actual = subject.move(4, 5)
      expect(actual.position).to eq([4, 5])
    end

    it "does not allow negative values" do
      actual = subject.move(-4, 5)
      expect(actual.position).to eq([0, 5])

      actual = subject.move(5, -4)
      expect(actual.position).to eq([5, 0])
    end
  end

  describe "#position" do
    let(:line) { 1 }
    let(:column) { 2 }

    it "returns an array with the line and column" do
      expect(subject.position).to eq([1, 2])
    end
  end

  describe "#right" do
    it "moves cursor right" do
      actual = subject.right
      expect(actual.position).to eq([1, 3])

      actual = subject.right(2)
      expect(actual.position).to eq([1, 4])
    end
  end

  describe "#up" do
    it "moves cursor up" do
      actual = subject.up
      expect(actual.position).to eq([0, 2])

      actual = subject.up(2)
      expect(actual.position).to eq([0, 2])
    end
  end
end
